#ifndef RAWDATABLOCKHEADER_H
#define RAWDATABLOCKHEADER_H
#include "DAQDataFormats/RawDataBlockHeaderV2.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawDataBlockHeader
#define  LAST_CLASS_VERSION           2
#define  CURRENT_FIELDS_SUB_NAMESPACE datablockheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace datablockheader {
        static const version_t RAWDATABLOCKHEADER_LATEST_VERSION = LAST_CLASS_VERSION;
    }

    class RawDataBlockHeader: public PREVIOUS_VERSION_CLASS_NAME{
        public:
      
            /// Constructors
            RawDataBlockHeader (const version_t version = datablockheader::RAWDATABLOCKHEADER_LATEST_VERSION);                             //! The default constructor
            RawDataBlockHeader (const RawDataBlockHeader& copy_in);  //! Deep copy constructor
            RawDataBlockHeader (const void* dadablockheader_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION); //! Shallow copy constructor
            /// Destructor
            ~RawDataBlockHeader();

            IMPLEMENT_MAIN_STANDARD

            IMPLEMENT_GET_MAIN        (Version       , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(Marker        , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(TrigNumLo     , uint32_t);
            IMPLEMENT_GET_AND_SET_MAIN(TrigNumHi     , uint32_t);
            IMPLEMENT_GET_AND_SET_MAIN(NumMicroBlocks, uint16_t)
            IMPLEMENT_GET_AND_SET_MAIN(BuffId        , uint16_t)
            IMPLEMENT_GET_AND_SET_MAIN(DataSize      , uint32_t)
            IMPLEMENT_GET_AND_SET_MAIN(MonteCarloFlag, bool)

            IMPLEMENT_IS_AND_SET_MAIN(CRCCalculationUsed)
            IMPLEMENT_IS_AND_SET_MAIN(ConnectionInitialization)
            IMPLEMENT_IS_AND_SET_MAIN(CloseConnection)
            IMPLEMENT_IS_AND_SET_MAIN(MissingData)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getTrigNum           , const, uint64_t )
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setTrigNum           ,      , bool, const uint64_t )
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(addDataSize          ,      , bool, const uint32_t )
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(advanceNumMicroBlocks,      , bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(checkMarker          , const, bool)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawDataBlockHeader)

        protected:
            IMPLEMENT_SET_MAIN        (Version, uint32_t)
            bool                      setFunctionPointers();

    };
 
  
} // END NAMESPACE daqdataformats
#endif /* RAWDATABLOCKHEADER_H */
