#ifndef RAWEVENT_H
#define RAWEVENT_H
#include "DAQDataFormats/RawEventV1.h"
#include "DAQDataFormats/RawEventHeader.h"
#include "DAQDataFormats/RawEventTail.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEvent
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawevent
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    //--------------------------------------------------------------------------
    /// Function to get the tail version from Event version
    inline version_t getTailVersion(const version_t event_version){
        
        if(event_version == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
            return rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION;

        const RawEventHeader header(event_version);
        return header.getTailVersion();
    }

    //--------------------------------------------------------------------------
    /// Function to get the tail size from Event version
    inline uint32_t getTailSize(const version_t event_version){
        const version_t event_tail_version = getTailVersion(event_version);

        if(event_tail_version == rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION)
            return 0;

        const RawEventTail tail(event_tail_version);
        return tail.sizeofdata();
    }

    static const uint32_t DEFAULT_EVENT_SIZE = 262144; /* Maximum MicroBlock Buffer Size 64k */

    class RawEvent : public PREVIOUS_VERSION_CLASS_NAME {

        public:
            /// Constructors
            RawEvent (const uint32_t buffersize = DEFAULT_EVENT_SIZE, const version_t version=raweventhdr::RAWEVENTHEADER_LATEST_VERSION);                        //! The default constructor
            RawEvent (const RawEvent& copy_in);  //! Deep copy constructor
            RawEvent (const void* datablock_buffer, const version_t version = rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION);       //! Shallow copy constructor
            /// Destructor
            ~RawEvent();

            IMPLEMENT_MAIN_STANDARD
            IMPLEMENT_MAIN_setBufferSource

            bool addDataBlock (const void* buffer)                     { RETURN_FUNCTION(addDataBlock)(buffer);}
            bool addDataBlock (const void* buffer, const uint32_t size){ RETURN_FUNCTION(addDataBlockWithSize)(buffer, size);}
            bool close (const bool produce_crc=false)                  {RETURN_FUNCTION(close)(produce_crc);}
            bool setTrigger(const daqdataformats::RawTrigger& tr)      {RETURN_FUNCTION(setTrigger1)(tr);}
            bool setTrigger(const void* buffer)                        {RETURN_FUNCTION(setTrigger2)(buffer);}

            /// Next event. Advance the event number and clear the DataBlocks and CRC
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(nextEvent           ,  , bool)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getHeader           , const, daqdataformats::RawEventHeader*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getTail             , , daqdataformats::RawEventTail*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getTrigger          , , daqdataformats::RawTrigger*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getFloatingDataBlock, , daqdataformats::RawDataBlock*)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getHeaderMarkerLeft , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getHeaderMarkerRight, const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getVersion          , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getDataLoggerID     , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getRunNumber        , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getRunType          , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getConfigIDX        , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getPartition        , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getSubrun           , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getEventNumberLo    , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getEventNumberHi    , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getEventNumber      , const, uint64_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getEventType        , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getEventSize        , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getDataBlockNumber  , const, uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(isEventIncomplete   , const, bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getMonteCarloFlag   , const, bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(isCRCCalculationUsed, const, bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setTriggerPointer   ,      , bool, const void*)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setDataLoggerID    , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setRunNumber       , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setRunType         , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setConfigIDX       , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setPartition       , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setSubrun          , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setEventNumberLo   , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setEventNumberHi   , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setEventNumber     , , bool, const uint64_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setEventType       , , bool, const uint32_t)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setEventIncomplete , , bool, const bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setMonteCarloFlag  , , bool, const bool)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setFloatingDataBlock, const ,bool, const uint32_t)

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawEventHeader)

        protected:
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getHeaderConst           , const, daqdataformats::RawEventHeader*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getTailConst             , const, daqdataformats::RawEventTail*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getTriggerConst          , const, daqdataformats::RawTrigger*)
            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_0PAR(getFloatingDataBlockConst, const, daqdataformats::RawDataBlock*)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setAppropriateMCFlag,  , bool, const void*)

            IMPLEMENT_FUNCTION_RETURNING_FUNCTION_POINTER_1PAR(setFloatingDataBlockToBufferSource, const, bool, const void*)

            bool setFunctionPointers();


    };// end of class definition

} // END NAMESPCE daqdataformats
#endif /* RAWEVENT_H */
