#ifndef RAWRUNHEADERV0_H
#define RAWRUNHEADERV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

// It is requred to
//   1. include DAQDataFormats/DataFormatFirst.inc
//   2. define CURRENT_CLASS
//   3.    define CURRENT_CLASS_VERSION for version of the class
//      OR define LAST_CLASS_VERSION    for the main structure of the class
//   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
//   5. include DAQDataFormats/DataFormatLast.inc
// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawRunHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawrunhdr
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        // Run header delimiter
        static const uint32_t Marker_value[2] = {0x41764F4E, 0x39323945};

    }

    namespace rawruntail {
        // Run tail delimiters
        static const uint32_t Marker_value[4] = {0x4C494154, 0x4C494154, 0x39323945, 0x41764F4E};
    }

  namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t RUNHEADER_SIZE = 25;

            enum RunHeaderMASKS{
                MASK_MarkerHi                 = 0xFFFFFFFF, //! Marker Mask upper word (32 bits)
                MASK_MarkerLo                 = 0xFFFFFFFF, //! Marker Mask lower word (32 bits)
                MASK_RunNumber                = 0x00FFFFFF, //! Run number             (24 bits)
                MASK_RunType                  = 0xFF000000, //! Run type               (8 bits)
                MASK_ConfigIDX                = 0x0000FFFF, //! Detector configuration identifier (16 bits)
                MASK_Partition                = 0x00FF0000, //! Partition identifier that Run Control and Data Logger belong to (8 bits)
                MASK_Subrun                   = 0x3F000000, //! Subrun number          (6 bits)
                MASK_ConfigurationBlockSet    = 0x80000000, //! Is the configuration Block set? (1bit)
                MASK_Version                  = 0x000000FF, //! Run Header Version     (8 bits)
                MASK_RunControlVersion        = 0x0000FF00, //! Run Control version    (8 bits)
                MASK_RunControlID             = 0xFFFF0000, //! Run control identifier (16 bits)
                MASK_DataLoggerVersion        = 0x00000FFF, //! Data Logger version    (12 bits)
                MASK_DataLoggerID             = 0xFFFF0000, //! Data Logger identifier (16 bits)
                MASK_RunStartTime             = 0xFFFFFFFF, //! Run Start Time         (32 bits)
                MASK_SubRunStartTime          = 0xFFFFFFFF, //! SubRun Start Time      (32 bits)
                MASK_SubRunEndTime            = 0xFFFFFFFF, //! SubRun End Time        (32 bits)
                MASK_RunEndTime               = 0xFFFFFFFF, //! Run End Time           (32 bits)
                MASK_TotalEvents              = 0xFFFFFFFF, //! Total number of events (32 bits)
                MASK_UniversalEventStartLo    = 0xFFFFFFFF, //! Universal event number lower 32 bits
                MASK_UniversalEventStartHi    = 0x000000FF, //! Universal event number higher 8 bits
                MASK_UniversalEventEndLo      = 0xFFFFFFFF, //! Universal event number lower 32 bits
                MASK_UniversalEventEndHi      = 0x000000FF, //! Universal event number higher 8 bits
                MASK_RunSize                  = 0xFFFFFFFF, //! Size of the run in words
                MASK_TriggerVersion           = 0x00000FFF, //! Trigger version (12 bits)
                MASK_TriggerCtrlID            = 0xFFFF0000, //! Identifier of the global trigger processor (16 bits)
                MASK_TriggerMC                = 0x80000000, //! MC triggering flag
                MASK_VTMod                    = 0x3F000000, //! Valid trigger modification modes
                MASK_ValidTriggerTypesLow     = 0x00FFFFFF, //! Logical OR of all valid trigger types. First word
                MASK_ValidTriggerTypesHigh    = 0xFFFFFFFF, //! Logical OR of all valid trigger types. Second word
                MASK_ValidTriggerTypesHigh2   = 0xFFFFFFFF, //! Logical OR of all valid trigger types. Third word
                MASK_TriggerListIDX           = 0xFFFFFFFF,
                MASK_TriggerPrescaleListIDX   = 0xFFFFFFFF,
                MASK_Reserved1                = 0xFFFFFFFF,
                MASK_Reserved2                = 0xFFFFFFFF,
                MASK_CRC                      = 0xFFFFFFFF
            };

            enum RunHeaderWORDS{
                WORD_MarkerHi                 = 0,  //! Marker Mask upper word (32 bits)
                WORD_MarkerLo                 = 1,  //! Marker Mask lower word (32 bits)
                WORD_RunNumber                = 2,  //! Run number             (24 bits)
                WORD_RunType                  = 2, //! Run type               (8 bits)
                WORD_ConfigIDX                = 3,  //! Detector configuration identifier (16 bits)
                WORD_Partition                = 3, //! Partition identifier that Run Control and Data Logger belong to (8 bits)
                WORD_Subrun                   = 3, //! Subrun number          (6 bits)
                WORD_ConfigurationBlockSet    = 3, //! Is the configuration Block set? (1bit)
                WORD_Version                  = 4, //! Run Header Version     (8 bits)
                WORD_RunControlVersion        = 4, //! Run Control version    (8 bits)
                WORD_RunControlID             = 4, //! Run control identifier (16 bits)
                WORD_DataLoggerVersion        = 5, //! Data Logger version    (12 bits)
                WORD_DataLoggerID             = 5, //! Data Logger identifier (16 bits)
                WORD_RunStartTime             = 6, //! Run Start Time         (32 bits)
                WORD_SubRunStartTime          = 7, //! SubRun Start Time      (32 bits)
                WORD_SubRunEndTime            = 8, //! SubRun End Time        (32 bits)
                WORD_RunEndTime               = 9, //! Run End Time           (32 bits)
                WORD_TotalEvents              = 10, //! Total number of events (32 bits)
                WORD_UniversalEventStartLo    = 11, //! Universal event number lower 32 bits
                WORD_UniversalEventStartHi    = 12, //! Universal event number higher 8 bits
                WORD_UniversalEventEndLo      = 13, //! Universal event number lower 32 bits
                WORD_UniversalEventEndHi      = 14, //! Universal event number higher 8 bits
                WORD_RunSize                  = 15, //! Size of the run in words
                WORD_TriggerVersion           = 16, //! Trigger version (12 bits)
                WORD_TriggerCtrlID            = 16,//! Identifier of the global trigger processor (16 bits)
                WORD_TriggerMC                = 17,//! MC triggering flag
                WORD_VTMod                    = 17,//! Valid trigger modification modes
                WORD_ValidTriggerTypesLow     = 17, //! Logical OR of all valid trigger types. First word
                WORD_ValidTriggerTypesHigh    = 18, //! Logical OR of all valid trigger types. Second word
                WORD_ValidTriggerTypesHigh2   = 19, //! Logical OR of all valid trigger types. Third word
                WORD_TriggerListIDX           = 20,
                WORD_TriggerPrescaleListIDX   = 21,
                WORD_Reserved1                = 22,
                WORD_Reserved2                = 23,
                WORD_CRC                      = 24
            };


                DEFINE_SHIFT_FROM_MASK(MarkerHi                 )
                DEFINE_SHIFT_FROM_MASK(MarkerLo                 )
                DEFINE_SHIFT_FROM_MASK(RunNumber                )
                DEFINE_SHIFT_FROM_MASK(RunType                  )
                DEFINE_SHIFT_FROM_MASK(ConfigIDX                )
                DEFINE_SHIFT_FROM_MASK(Partition                )
                DEFINE_SHIFT_FROM_MASK(Subrun                   )
                DEFINE_SHIFT_FROM_MASK(ConfigurationBlockSet    )
                DEFINE_SHIFT_FROM_MASK(Version                  )
                DEFINE_SHIFT_FROM_MASK(RunControlVersion        )
                DEFINE_SHIFT_FROM_MASK(RunControlID             )
                DEFINE_SHIFT_FROM_MASK(DataLoggerVersion        )
                DEFINE_SHIFT_FROM_MASK(DataLoggerID             )
                DEFINE_SHIFT_FROM_MASK(RunStartTime             )
                DEFINE_SHIFT_FROM_MASK(SubRunStartTime          )
                DEFINE_SHIFT_FROM_MASK(SubRunEndTime            )
                DEFINE_SHIFT_FROM_MASK(RunEndTime               )
                DEFINE_SHIFT_FROM_MASK(TotalEvents              )
                DEFINE_SHIFT_FROM_MASK(UniversalEventStartLo    )
                DEFINE_SHIFT_FROM_MASK(UniversalEventStartHi    )
                DEFINE_SHIFT_FROM_MASK(UniversalEventEndLo      )
                DEFINE_SHIFT_FROM_MASK(UniversalEventEndHi      )
                DEFINE_SHIFT_FROM_MASK(RunSize                  )
                DEFINE_SHIFT_FROM_MASK(TriggerVersion           )
                DEFINE_SHIFT_FROM_MASK(TriggerCtrlID            )
                DEFINE_SHIFT_FROM_MASK(TriggerMC                )
                DEFINE_SHIFT_FROM_MASK(VTMod                    )
                DEFINE_SHIFT_FROM_MASK(ValidTriggerTypesLow     )
                DEFINE_SHIFT_FROM_MASK(ValidTriggerTypesHigh    )
                DEFINE_SHIFT_FROM_MASK(ValidTriggerTypesHigh2   )
                DEFINE_SHIFT_FROM_MASK(TriggerListIDX           )
                DEFINE_SHIFT_FROM_MASK(TriggerPrescaleListIDX   )
                DEFINE_SHIFT_FROM_MASK(Reserved1                )
                DEFINE_SHIFT_FROM_MASK(Reserved2                )
                DEFINE_SHIFT_FROM_MASK(CRC                      )
        }// end of namespace rawrunhdr

  
  class RawRunHeader: public PREVIOUS_VERSION_CLASS_NAME {
        public:

            RawRunHeader (const version_t); ///< The default constructor
            RawRunHeader (const RawRunHeader&);
            ~RawRunHeader(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const {  return CURRENT_FIELDS_SUB_NAMESPACE::RUNHEADER_SIZE;}
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

        protected:

            IMPLEMENT_FIRST_GET        (MarkerHi              , uint32_t)
            IMPLEMENT_FIRST_SET        (MarkerHi              , uint32_t)
            IMPLEMENT_FIRST_GET        (MarkerLo              , uint32_t)
            IMPLEMENT_FIRST_SET        (MarkerLo              , uint32_t)
            IMPLEMENT_FIRST_GET        (Reserved1             , uint32_t)
            IMPLEMENT_FIRST_SET        (Reserved1             , uint32_t)
            IMPLEMENT_FIRST_GET        (Reserved2             , uint32_t)
            IMPLEMENT_FIRST_SET        (Reserved2             , uint32_t)
            IMPLEMENT_FIRST_GET        (Version               , uint32_t)
            IMPLEMENT_FIRST_SET        (Version               , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunNumber             , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunType               , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ConfigIDX             , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Partition             , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Subrun                , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ConfigurationBlockSet , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunControlVersion     , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunControlID          , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(DataLoggerVersion     , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(DataLoggerID          , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunStartTime          , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(SubRunStartTime       , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(SubRunEndTime         , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunEndTime            , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TotalEvents           , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(UniversalEventStartLo , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(UniversalEventStartHi , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(UniversalEventEndLo   , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(UniversalEventEndHi   , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(RunSize               , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerVersion        , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerCtrlID         , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerMC             , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(VTMod                 , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ValidTriggerTypesLow  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ValidTriggerTypesHigh , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ValidTriggerTypesHigh2, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerListIDX        , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerPrescaleListIDX, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(CRC                   , uint32_t)

	    INIT_FIRST(getUniversalEventStart, const,uint64_t, )
            INIT_FIRST(setUniversalEventStart, ,bool    , const uint64_t)
	    INIT_FIRST(getUniversalEventEnd  , const,uint64_t, )
            INIT_FIRST(setUniversalEventEnd  , ,bool    , const uint64_t)

	    INIT_FIRST(setTail               , ,bool, )
	    INIT_FIRST(close                 , ,bool, )
	    INIT_FIRST(advanceEventNumber    , ,bool, )

            INIT_FIRST(addValidTriggerTypesLow  , ,bool    , const uint32_t)
            INIT_FIRST(addValidTriggerTypesHigh , ,bool    , const uint32_t)
            INIT_FIRST(addValidTriggerTypesHigh2, ,bool    , const uint32_t)
            INIT_FIRST(addRunSize               , ,bool    , const uint32_t)

  };
  
  
  }} // END NAMESPCE daqdataformats
#endif /* RAWRUNHEADERV0_H */
