#ifndef RAWSUMMARYDCMDATA_H
#define RAWSUMMARYDCMDATA_H
#include "DAQDataFormats/RawSummaryDCMDataV0.h"
#include "DAQDataFormats/RawSummaryDCMDataPoint.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryDCMData
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE summarydcmdata
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

  static const uint32_t DEFAULT_DCMDATA_SIZE = 2; /* Default MicroBlock Buffer Size  */

  class RawSummaryDCMData : public PREVIOUS_VERSION_CLASS_NAME {
        public:
            // Constructors
    RawSummaryDCMData (const version_t version=summarydcmdatahdr::RAWSUMMARYDCMDATA_LATEST_VERSION); //! The default constructor
            RawSummaryDCMData (const RawSummaryDCMData& copy_in);                  //! Deep copy constructor
            RawSummaryDCMData (const void* dcmdata_buffer, const version_t version=rawdaqdataformat::DAQDATAFORMAT_UNKNOWN_VERSION, const uint32_t buffersize = DEFAULT_DCMDATA_SIZE); //! Shallow copy constructor
            /// Destructor
            ~RawSummaryDCMData();

            IMPLEMENT_MAIN_STANDARD
            IMPLEMENT_MAIN_setBufferSource
            
            IMPLEMENT_GET_MAIN(Version, uint32_t);

            IMPLEMENT_FIGUREOUT_VERSION_STATIC(RawSummaryDCMDataHeader)

	      bool addMissingDCMs(const std::list< std::pair<uint64_t, uint16_t> >& dcmlist)
	    {RETURN_FUNCTION(addDataPoints)(dcmlist, DCMDataPointType_MissingDCM); }
	    bool addMissingMicroslices(const std::list< std::pair<uint64_t, uint16_t> >& mslist)
	    {RETURN_FUNCTION(addDataPoints)(mslist, DCMDataPointType_MissingMicroslice); }
	     bool  addCorruptMicroslices(const std::list< std::pair<uint64_t, uint16_t> >& mslist)
	    {RETURN_FUNCTION(addDataPoints)(mslist, DCMDataPointType_CorruptMicroslice); }

	     bool addDataPoints(const std::list< std::pair<uint64_t, uint16_t> >& list, uint8_t type)
	     {return functionIsNotAllowed(__FUNCTION__, "Function is private."); }
        private:
            bool setFunctionPointers();

  };

} // END NAMESPCE daqdataformats
#endif /* RAWSUMMARYDCMDATA_H */
