#ifndef __DBROW_HPP_
#define __DBROW_HPP_

#include <string>
#include <vector>
#include <iostream>

#include "NovaDatabase/NovaDatabase.h"
#include "NovaDatabase/Column.h"

namespace nova {
  namespace database {

    /**
     * Generalized Database Row Interface
     *
     * @author Jonathan Paley
     * @version $Id: Row.h,v 1.17 2016/05/03 03:05:02 jpaley Exp $
     */
    class Row : public dBRow_t
    {
    public:
      Row(const std::vector<Column>&, const dBRow_t&);
      Row(const std::vector<Column>&);
      ~Row();
      
      void    Clear();
      
      template <class T>      
	bool    Set(std::string cname, T value) {
	Column* col = this->Col(cname);
	if (!col) return false;	
	return (col->Set(value));
      }

      template <class T>
	bool    Update(std::string cname, T value) {
	Column* col = this->Col(cname);
	if (!col) return false;	
	if (! col->Modified())   fNModified++;
	return (col->Update(value));
      }
      bool    InDB() { return fInDB; }
      void    SetInDB() { fInDB = true; }

      int     NModified() { return fNModified; }
      void    ClearModified() { fNModified = 0; }

      int     NCol() { return fCol.size(); }

      Column& Col(int i) {return fCol[i]; }
      Column* Col(std::string name);
      Column* Col(const char* name) { return Col(std::string(name)); }

      //      bool operator==(const Row& other) const;

      friend std::ostream& operator<< (std::ostream& stream, const Row& row);
      //      friend std::istream& operator>> (std::istream& stream, Row& row);

    private:
      bool                fInDB;
      int                 fNModified;
      std::vector<Column> fCol;

    }; // class end

    //************************************************************
    
    inline std::ostream& operator<< (std::ostream& stream, const Row& row) { 
      for (unsigned int i=0; i<row.fCol.size(); ++i) {
	stream << row.fCol[i];
	if (i < row.fCol.size()-1)
	  stream << ", ";
      }      
      return stream;
    }
    
  } // namespace novadaqdatabase close
} // namespace nova close

#endif
