#include "DAQDataFormats/RawNanoSliceHeaderV0.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSliceHeader
#define  CURRENT_CLASS_VERSION        0
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawNanoSliceHeader::RawNanoSliceHeader(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
    /*
    ,ZEROOUT_GET_AND_SET(FEBId)
    ,ZEROOUT_GET_AND_SET(FEBStatus)
    ,ZEROOUT_GET_AND_SET(PixelAddr)
    ,ZEROOUT_GET_AND_SET(Sequence)
    ,ZEROOUT_GET_AND_SET(Version)
    ,ZEROOUT_GET_AND_SET(Pad3)
    ,ZEROOUT_GET_AND_SET(DataPresent)
    ,ZEROOUT_GET_AND_SET(LinkStatus)
    ,ZEROOUT_GET_AND_SET(DebugMode)
    ,ZEROOUT_GET_AND_SET(DAQEnabled)
    ,ZEROOUT_GET_AND_SET(BufferEmpty)
    ,ZEROOUT_GET_AND_SET(BufferFull)
    ,ZEROOUT_GET_AND_SET(CommError)
    ,ZEROOUT_GET_AND_SET(PacketError)
    ,ZEROOUT_GET_AND_SET(OverflowError)
    ,ZEROOUT_GET_AND_SET(ADCError)
    */
    ,ZEROOUT_FUNCTION(initWithData)
    ,ZEROOUT_FUNCTION(setParameters)
{}

RawNanoSliceHeader::RawNanoSliceHeader(const RawNanoSliceHeader& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    /*
    ,DEEPCOPY_GET_AND_SET(FEBId)
    ,DEEPCOPY_GET_AND_SET(FEBStatus)
    ,DEEPCOPY_GET_AND_SET(PixelAddr)
    ,DEEPCOPY_GET_AND_SET(Sequence)
    ,DEEPCOPY_GET_AND_SET(Version)
    ,DEEPCOPY_GET_AND_SET(Pad3)
    ,DEEPCOPY_GET_AND_SET(DataPresent)
    ,DEEPCOPY_GET_AND_SET(LinkStatus)
    ,DEEPCOPY_GET_AND_SET(DebugMode)
    ,DEEPCOPY_GET_AND_SET(DAQEnabled)
    ,DEEPCOPY_GET_AND_SET(BufferEmpty)
    ,DEEPCOPY_GET_AND_SET(BufferFull)
    ,DEEPCOPY_GET_AND_SET(CommError)
    ,DEEPCOPY_GET_AND_SET(PacketError)
    ,DEEPCOPY_GET_AND_SET(OverflowError)
    ,DEEPCOPY_GET_AND_SET(ADCError)
    */
    ,DEEPCOPY_FUNCTION(initWithData)
    ,DEEPCOPY_FUNCTION(setParameters)
{}

//------------------------Destructor--------------------------------------------
RawNanoSliceHeader::~RawNanoSliceHeader() { }

//------------------------------------------------------------------------------
// Default initialization at wherever the pointer points
void RawNanoSliceHeader::init(){

    /// Set version in the buffer
    //EXECUTE_FUNCTION(setVersion)(_version);
    setVersion(_version);

    /// Set Pad3 in the buffer do the defalut value
    //EXECUTE_FUNCTION(setPad3)(nanosliceheader::PAD3_DEFAULT_VALUE);
    setPad3(nanosliceheader::PAD3_DEFAULT_VALUE);
}

//------------------------------------------------------------------------------
// Initialization with data
 void RawNanoSliceHeader::initWithData(const NanoSliceHeader& pars){

   /// Set all parameters in the buffer, including version
   EXECUTE_FUNCTION(setParameters)(pars);

   /// Set Pad3 do the defalut value
   //EXECUTE_FUNCTION(setPad3)(nanosliceheader::PAD3_DEFAULT_VALUE);
   setPad3(nanosliceheader::PAD3_DEFAULT_VALUE);
 }

//------------------------------------------------------------------------------
bool RawNanoSliceHeader::setParameters(const NanoSliceHeader& pars){

        /*
        EXECUTE_FUNCTION(setLinkStatus ) (pars.LinkStatus);
        EXECUTE_FUNCTION(setDataPresent) (pars.DataPresent);
        EXECUTE_FUNCTION(setDebugMode  ) (pars.DebugMode);
        EXECUTE_FUNCTION(setFEBId      ) (pars.FEBLinkID);
        EXECUTE_FUNCTION(setFEBStatus  ) (pars.FEBStatus);
        EXECUTE_FUNCTION(setPixelAddr  ) (pars.PixelAddress);
        EXECUTE_FUNCTION(setSequence   ) (pars.FrameSequenceNumber);
        EXECUTE_FUNCTION(setVersion    ) (pars.Version);
        */
        setLinkStatus    (pars.LinkStatus);
        setDataPresent   (pars.DataPresent);
        setDebugMode     (pars.DebugMode);
        setFEBId         (pars.FEBLinkID);
        setFEBStatus     (pars.FEBStatus);
        setPixelAddr     (pars.PixelAddress);
        setSequence      (pars.FrameSequenceNumber);
        return setVersion(pars.Version);
}

//------------------------------------------------------------------------------
bool RawNanoSliceHeader::advanceSequence()
{ 
    //RETURN_FUNCTION(setSequence)(EXECUTE_FUNCTION(getSequence)() + 1);
    return setSequence(getSequence() + 1);
}

//------------------------------------------------------------------------------
// Printing method
void RawNanoSliceHeader::print(std::ostream& os) const {

    os<<boost::format("NanoSlice Header: LS=%d DP=%d DM=%d FEBID=%u FEBSTAT=%u PIXEL=%u VERSION=%u SEQ=%u\n")
           /*
           % EXECUTE_FUNCTION(getLinkStatus)()
	   % EXECUTE_FUNCTION(getDataPresent)()
	   % EXECUTE_FUNCTION(getDebugMode)()
	   % EXECUTE_FUNCTION(getFEBId)()
	   % EXECUTE_FUNCTION(getFEBStatus)()
	   % EXECUTE_FUNCTION(getPixelAddr)()
	   % EXECUTE_FUNCTION(getVersion)()
	   % EXECUTE_FUNCTION(getSequence)();
           */
           % getLinkStatus()
	   % getDataPresent()
	   % getDebugMode()
	   % getFEBId()
	   % getFEBStatus()
	   % getPixelAddr()
	   % getVersion()
	   % getSequence();

            printBuffer(os);
            os<<"RAW(binary):";
            printWord(0,true, os);
            os<<std::endl;
}// end of print1

}}// end of daqdataformats::version0