#include "DAQDataFormats/RawNanoSliceV1.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawNanoSlice
#define  CURRENT_CLASS_VERSION        1
#define  CURRENT_FIELDS_SUB_NAMESPACE rawnano
#include "DAQDataFormats/DataFormatLast.inc"
  
namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------Constructors------------------------------------------
RawNanoSlice::RawNanoSlice(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Value1)
{ }
RawNanoSlice::RawNanoSlice(const NanoSlice& nano):
             PREVIOUS_VERSION_CLASS_NAME(nano)
            ,ZEROOUT_GET_AND_SET(Value1)
{ }
//------------------------------------------------------------------------------
RawNanoSlice::RawNanoSlice (const RawNanoSlice& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Value1)
{}

//------------------------Destructor--------------------------------------------
RawNanoSlice::~RawNanoSlice() { }


//------------------------------------------------------------------------------
// Default Initialization of Data members
void RawNanoSlice::init(){
    /// After using resetBufferSource header is already constructed
    /// Now we need to initialize it
    _Header->init();
}

//------------------------------------------------------------------------------
void RawNanoSlice::initWithData(const NanoSlice& nanoslice_parameters){

    /// After using resetBufferSource header is already constructed
    /// Now we need to initialize it
    _Header->initWithData(nanoslice_parameters.Header);

    EXECUTE_FUNCTION(setTimeStamp)(nanoslice_parameters.TimeStamp);
    EXECUTE_FUNCTION(setValue1)(nanoslice_parameters.Value1);
    
}// end of initWithData

//-----------------------------------------------------------------
// Set all parameters
bool RawNanoSlice::setParameters(const NanoSlice& nanoslice_parameters){
  
        _Header->setParameters(nanoslice_parameters.Header);

        EXECUTE_FUNCTION(setTimeStamp)(nanoslice_parameters.TimeStamp);
        EXECUTE_FUNCTION(setValue1)(nanoslice_parameters.Value1);

        return true;
}

//------------------------------------------------------------------------------
// Printing the NanoSlice
void RawNanoSlice::print(std::ostream& os) const{

    if(_version == 0) return version0::RawNanoSlice::print(os);

    _Header->print(os);

    os<<boost::format(" NanoSlice Body: TimeStamp=%u")
	 %EXECUTE_FUNCTION(getTimeStamp)()
	 ;

    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<"\nsize of data = "<<size_of_data<<"\n";


    os<<"\nRAW(hex):";
    for(uint32_t i = 0; i < size_of_data; ++i){
            printWord(i, false, os);
            os<<" ";
    }

    os<<"\nRAW(binary):";
    for(uint32_t i = 0; i < size_of_data; ++i){
            if(i > 0)os<<"    ";
            printWord(i, true, os);
            os<<std::endl;
    }

}// end of print

}}// end of namespaces
