#include "DAQDataFormats/RawRunHeaderV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

// It is requred to
//   1. include DAQDataFormats/DataFormatFirst.inc
//   2. define CURRENT_CLASS
//   3.    define CURRENT_CLASS_VERSION for version of the class
//      OR define LAST_CLASS_VERSION    for the main structure of the class
//   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
//   5. include DAQDataFormats/DataFormatLast.inc
// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawRunHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawrunhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawRunHeader::RawRunHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
    ,ZEROOUT_GET        (MarkerHi              )
    ,ZEROOUT_SET        (MarkerHi              )
    ,ZEROOUT_GET        (MarkerLo              )
    ,ZEROOUT_SET        (MarkerLo              )
    ,ZEROOUT_GET        (Reserved1             )
    ,ZEROOUT_SET        (Reserved1             )
    ,ZEROOUT_GET        (Reserved2             )
    ,ZEROOUT_SET        (Reserved2             )
    ,ZEROOUT_GET        (Version               )
    ,ZEROOUT_SET        (Version               )
    ,ZEROOUT_GET_AND_SET(RunNumber             )
    ,ZEROOUT_GET_AND_SET(RunType               )
    ,ZEROOUT_GET_AND_SET(ConfigIDX             )
    ,ZEROOUT_GET_AND_SET(Partition             )
    ,ZEROOUT_GET_AND_SET(Subrun                )
    ,ZEROOUT_GET_AND_SET(ConfigurationBlockSet )
    ,ZEROOUT_GET_AND_SET(RunControlVersion     )
    ,ZEROOUT_GET_AND_SET(RunControlID          )
    ,ZEROOUT_GET_AND_SET(DataLoggerVersion     )
    ,ZEROOUT_GET_AND_SET(DataLoggerID          )
    ,ZEROOUT_GET_AND_SET(RunStartTime          )
    ,ZEROOUT_GET_AND_SET(SubRunStartTime       )
    ,ZEROOUT_GET_AND_SET(SubRunEndTime         )
    ,ZEROOUT_GET_AND_SET(RunEndTime            )
    ,ZEROOUT_GET_AND_SET(TotalEvents           )
    ,ZEROOUT_GET_AND_SET(UniversalEventStartLo )
    ,ZEROOUT_GET_AND_SET(UniversalEventStartHi )
    ,ZEROOUT_GET_AND_SET(UniversalEventEndLo   )
    ,ZEROOUT_GET_AND_SET(UniversalEventEndHi   )
    ,ZEROOUT_GET_AND_SET(RunSize               )
    ,ZEROOUT_GET_AND_SET(TriggerVersion        )
    ,ZEROOUT_GET_AND_SET(TriggerCtrlID         )
    ,ZEROOUT_GET_AND_SET(TriggerMC             )
    ,ZEROOUT_GET_AND_SET(VTMod                 )
    ,ZEROOUT_GET_AND_SET(ValidTriggerTypesLow  )
    ,ZEROOUT_GET_AND_SET(ValidTriggerTypesHigh )
    ,ZEROOUT_GET_AND_SET(ValidTriggerTypesHigh2)
    ,ZEROOUT_GET_AND_SET(TriggerListIDX        )
    ,ZEROOUT_GET_AND_SET(TriggerPrescaleListIDX)
    ,ZEROOUT_GET_AND_SET(CRC                   )
    ,ZEROOUT_FUNCTION(getUniversalEventStart)
    ,ZEROOUT_FUNCTION(setUniversalEventStart)
    ,ZEROOUT_FUNCTION(getUniversalEventEnd  )
    ,ZEROOUT_FUNCTION(setUniversalEventEnd  )
    ,ZEROOUT_FUNCTION(setTail               )
    ,ZEROOUT_FUNCTION(close                 )
    ,ZEROOUT_FUNCTION(advanceEventNumber    )
    ,ZEROOUT_FUNCTION(addValidTriggerTypesLow  )
    ,ZEROOUT_FUNCTION(addValidTriggerTypesHigh )
    ,ZEROOUT_FUNCTION(addValidTriggerTypesHigh2)
    ,ZEROOUT_FUNCTION(addRunSize               )
{}

//------------------------------------------------------------------------------
RawRunHeader::RawRunHeader(const RawRunHeader& copyin):
     PREVIOUS_VERSION_CLASS_NAME(copyin)
    ,DEEPCOPY_GET        (MarkerHi              )
    ,DEEPCOPY_SET        (MarkerHi              )
    ,DEEPCOPY_GET        (MarkerLo              )
    ,DEEPCOPY_SET        (MarkerLo              )
    ,DEEPCOPY_GET        (Reserved1             )
    ,DEEPCOPY_SET        (Reserved1             )
    ,DEEPCOPY_GET        (Reserved2             )
    ,DEEPCOPY_SET        (Reserved2             )
    ,DEEPCOPY_GET        (Version               )
    ,DEEPCOPY_SET        (Version               )
    ,DEEPCOPY_GET_AND_SET(RunNumber             )
    ,DEEPCOPY_GET_AND_SET(RunType               )
    ,DEEPCOPY_GET_AND_SET(ConfigIDX             )
    ,DEEPCOPY_GET_AND_SET(Partition             )
    ,DEEPCOPY_GET_AND_SET(Subrun                )
    ,DEEPCOPY_GET_AND_SET(ConfigurationBlockSet )
    ,DEEPCOPY_GET_AND_SET(RunControlVersion     )
    ,DEEPCOPY_GET_AND_SET(RunControlID          )
    ,DEEPCOPY_GET_AND_SET(DataLoggerVersion     )
    ,DEEPCOPY_GET_AND_SET(DataLoggerID          )
    ,DEEPCOPY_GET_AND_SET(RunStartTime          )
    ,DEEPCOPY_GET_AND_SET(SubRunStartTime       )
    ,DEEPCOPY_GET_AND_SET(SubRunEndTime         )
    ,DEEPCOPY_GET_AND_SET(RunEndTime            )
    ,DEEPCOPY_GET_AND_SET(TotalEvents           )
    ,DEEPCOPY_GET_AND_SET(UniversalEventStartLo )
    ,DEEPCOPY_GET_AND_SET(UniversalEventStartHi )
    ,DEEPCOPY_GET_AND_SET(UniversalEventEndLo   )
    ,DEEPCOPY_GET_AND_SET(UniversalEventEndHi   )
    ,DEEPCOPY_GET_AND_SET(RunSize               )
    ,DEEPCOPY_GET_AND_SET(TriggerVersion        )
    ,DEEPCOPY_GET_AND_SET(TriggerCtrlID         )
    ,DEEPCOPY_GET_AND_SET(TriggerMC             )
    ,DEEPCOPY_GET_AND_SET(VTMod                 )
    ,DEEPCOPY_GET_AND_SET(ValidTriggerTypesLow  )
    ,DEEPCOPY_GET_AND_SET(ValidTriggerTypesHigh )
    ,DEEPCOPY_GET_AND_SET(ValidTriggerTypesHigh2)
    ,DEEPCOPY_GET_AND_SET(TriggerListIDX        )
    ,DEEPCOPY_GET_AND_SET(TriggerPrescaleListIDX)
    ,DEEPCOPY_GET_AND_SET(CRC                   )
    ,DEEPCOPY_FUNCTION(getUniversalEventStart)
    ,DEEPCOPY_FUNCTION(setUniversalEventStart)
    ,DEEPCOPY_FUNCTION(getUniversalEventEnd  )
    ,DEEPCOPY_FUNCTION(setUniversalEventEnd  )
    ,DEEPCOPY_FUNCTION(setTail               )
    ,DEEPCOPY_FUNCTION(close                 )
    ,DEEPCOPY_FUNCTION(advanceEventNumber    )
    ,DEEPCOPY_FUNCTION(addValidTriggerTypesLow  )
    ,DEEPCOPY_FUNCTION(addValidTriggerTypesHigh )
    ,DEEPCOPY_FUNCTION(addValidTriggerTypesHigh2)
    ,DEEPCOPY_FUNCTION(addRunSize               )
{}

//------------------------------------------------------------------------------
RawRunHeader::~RawRunHeader(){ }

//------------------------------------------------------------------------------
// Initialization of data members
void RawRunHeader::init(){
    EXECUTE_FUNCTION(setVersion )(_version);
    EXECUTE_FUNCTION(setRunSize)(sizeofdata());
    EXECUTE_FUNCTION(setMarkerHi)(daqdataformats::rawrunhdr::Marker_value[0]);
    EXECUTE_FUNCTION(setMarkerLo)(daqdataformats::rawrunhdr::Marker_value[1]);
}


//------------------------------------------------------------------------------
uint64_t RawRunHeader::getUniversalEventStart () const         { 
    return getUint64_t(EXECUTE_FUNCTION(getUniversalEventStartLo)(), EXECUTE_FUNCTION(getUniversalEventStartHi)());
}
//------------------------------------------------------------------------------
bool     RawRunHeader::setUniversalEventStart (const uint64_t event) { 
      uint32_t low, high;
      getLoAndHi(event, low, high);

      EXECUTE_FUNCTION(setUniversalEventStartHi)(high);
      RETURN_FUNCTION(setUniversalEventStartLo)(low);
}
//------------------------------------------------------------------------------
uint64_t RawRunHeader::getUniversalEventEnd   () const         { 
    return getUint64_t(       EXECUTE_FUNCTION(getUniversalEventEndLo)()  , EXECUTE_FUNCTION(getUniversalEventEndHi)());}
//------------------------------------------------------------------------------
bool     RawRunHeader::setUniversalEventEnd   (const uint64_t event) { 
      uint32_t low, high;
      getLoAndHi(event, low, high);

      EXECUTE_FUNCTION(setUniversalEventEndHi)(high);
      RETURN_FUNCTION(setUniversalEventEndLo)(low);
}

//------------------------------------------------------------------------------
bool RawRunHeader::addValidTriggerTypesLow  (const uint32_t w){
    /// Add by applying a bitwize OR
    const uint32_t word = w | EXECUTE_FUNCTION(getValidTriggerTypesLow)();
    return setValidTriggerTypesLow(word);
}
//------------------------------------------------------------------------------
bool RawRunHeader::addValidTriggerTypesHigh (const uint32_t w){
    /// Add by applying a bitwize OR
    const uint32_t word = w | EXECUTE_FUNCTION(getValidTriggerTypesHigh)();
    RETURN_FUNCTION(setValidTriggerTypesHigh)(word);
}
//------------------------------------------------------------------------------
bool RawRunHeader::addValidTriggerTypesHigh2(const uint32_t w){
    /// Add by applying a bitwize OR
    const uint32_t word = w | EXECUTE_FUNCTION(getValidTriggerTypesHigh2)();
    RETURN_FUNCTION(setValidTriggerTypesHigh2)(word);
}

//------------------------------------------------------------------------------
bool RawRunHeader::addRunSize (const uint32_t size){ RETURN_FUNCTION(setRunSize)(EXECUTE_FUNCTION(getRunSize)() + size);}
//------------------------------------------------------------------------------
bool RawRunHeader::advanceEventNumber()      { RETURN_FUNCTION(setTotalEvents)(EXECUTE_FUNCTION(getTotalEvents)() + 1);}

//------------------------------------------------------------------------------
bool RawRunHeader::setTail(){
              EXECUTE_FUNCTION(setMarkerHi )(rawruntail::Marker_value[0]);
              EXECUTE_FUNCTION(setMarkerLo )(rawruntail::Marker_value[1]);
              EXECUTE_FUNCTION(setReserved1)(rawruntail::Marker_value[2]);
              RETURN_FUNCTION(setReserved2)(rawruntail::Marker_value[3]);
}

//------------------------------------------------------------------------------
bool RawRunHeader::close(){ RETURN_FUNCTION(setCRC)(calculateCheckSum(1));}

//------------------------------------------------------------------------------
// Printing method
void RawRunHeader::print(std::ostream& os) const{

    os<<boost::format("RawRun Header-------------------------------------- version %d\n")
            % getVersion();

    os<<boost::format(" RunNumber=%u")              % EXECUTE_FUNCTION(getRunNumber)();
    os<<boost::format(" RunType=%u")                % EXECUTE_FUNCTION(getRunType)();
    os<<boost::format(" ConfigIDX=%u")              % EXECUTE_FUNCTION(getConfigIDX)();
    os<<boost::format(" Partition=%u")              % EXECUTE_FUNCTION(getPartition)();
    os<<boost::format(" Subrun=%u")                 % EXECUTE_FUNCTION(getSubrun)();
    os<<boost::format(" ConfigBlockSet=%u")         % EXECUTE_FUNCTION(getConfigurationBlockSet)();
    os<<boost::format(" RunControlVersion=%u")      % EXECUTE_FUNCTION(getRunControlVersion)();
    os<<boost::format(" RunControlID=%u")           % EXECUTE_FUNCTION(getRunControlID)();
    os<<boost::format(" DataLoggerVersion=%u")      % EXECUTE_FUNCTION(getDataLoggerVersion)();
    os<<boost::format(" DataLoggerID=%u")           % EXECUTE_FUNCTION(getDataLoggerID)();
    os<<boost::format(" RunStartTime=%u")           % EXECUTE_FUNCTION(getRunStartTime)();
    os<<boost::format(" SubRunStartTime=%u")        % EXECUTE_FUNCTION(getSubRunStartTime)();
    os<<boost::format(" SubRunEndTime=%u")          % EXECUTE_FUNCTION(getSubRunEndTime )();
    os<<boost::format(" RunEndTime=%u")             % EXECUTE_FUNCTION(getRunEndTime)();
    os<<boost::format(" TotalEvents=%u")            % EXECUTE_FUNCTION(getTotalEvents)();
    os<<boost::format(" UniversalEventStartLo=%u")  % EXECUTE_FUNCTION(getUniversalEventStartLo)();
    os<<boost::format(" UniversalEventStartHi=%u")  % EXECUTE_FUNCTION(getUniversalEventStartHi)();
    os<<boost::format(" UniversalEventEndLo=%u")    % EXECUTE_FUNCTION(getUniversalEventEndLo)();
    os<<boost::format(" UniversalEventEndHi=%u")    % EXECUTE_FUNCTION(getUniversalEventEndHi)();
    os<<boost::format(" RunSize=%u")                % EXECUTE_FUNCTION(getRunSize)();
    os<<boost::format(" TriggerVersion=%u")         % EXECUTE_FUNCTION(getTriggerVersion)();
    os<<boost::format(" TriggerCtrlID=%u")          % EXECUTE_FUNCTION(getTriggerCtrlID)();
    os<<boost::format(" TriggerMC=%u")              % EXECUTE_FUNCTION(getTriggerMC)();
    os<<boost::format(" VTMod=%u")                  % EXECUTE_FUNCTION(getVTMod)();
    os<<boost::format(" ValidTriggerTypesLow=%u")   % EXECUTE_FUNCTION(getValidTriggerTypesLow)();
    os<<boost::format(" ValidTriggerTypesHigh=%u")  % EXECUTE_FUNCTION(getValidTriggerTypesHigh)();
    os<<boost::format(" ValidTriggerTypesHigh2=%u") % EXECUTE_FUNCTION(getValidTriggerTypesHigh2)();
    os<<boost::format(" TriggerListIDX=%u")         % EXECUTE_FUNCTION(getTriggerListIDX)();
    os<<boost::format(" TriggerPrescaleListIDX=%u") % EXECUTE_FUNCTION(getTriggerPrescaleListIDX)();

    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<" ";
        printWord(i,false, os);
    }

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
            if(i > 0)os<<"    ";
            printWord(i,true, os);
            os<<"\n";
            }

}


}}// end of namespaces