#include <NovaDAQUtilities/TimeUtils.h>
#include <boost/algorithm/string.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

namespace novadaq {

/**
 * Free function to convert a timeval struct into a double.
 */
double convertTimevalToDouble(struct timeval const& inputUnixTime)
{
    return ((double) inputUnixTime.tv_sec) +
        (((double) inputUnixTime.tv_usec) / 1000000.0);
}

/**
 * Free function to fetch the current time as a double.  (The
 * resulting time is still the number of seconds since the UNIX time
 * epoch, but it's represented as a double instead of a struct).
 */
double getCurrentTime()
{
    struct timeval unixTime;
    gettimeofday(&unixTime, 0);
    return convertTimevalToDouble(unixTime);
}

std::string formatDateTime(time_t secondsSinceTheEpoch)
{
    boost::posix_time::ptime convertedTime =
        boost::posix_time::from_time_t(secondsSinceTheEpoch);
    std::string resultString =
        boost::posix_time::to_iso_extended_string(convertedTime);

    // strip off any extra fractional seconds digits (more than three)
    size_t dotPos = resultString.find_last_of(".");
    if (dotPos != std::string::npos &&
        resultString.length() > (dotPos + 4)) {
        resultString = resultString.substr(0, (dotPos + 4));
    }
    dotPos = resultString.find_last_of(",");
    if (dotPos != std::string::npos &&
        resultString.length() > (dotPos + 4)) {
        resultString = resultString.substr(0, (dotPos + 4));
    }

    // replace the time separator with a space
    boost::replace_all(resultString, "T", " ");

    return resultString;
}

};  // end of namespace novadaq
